/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its center, with its center at the origin.
<p>
   Do this by "accumulating" rotations.
<p>
   Demonstrate the transformation of the coordinate system.
*/
public class Example_04d
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      scene.camera.projOrtho(left, right, bottom, top);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create two shorter sets of x and y axes.
      Model sAxes1 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      ModelShading.setColor(sAxes1, new Color(150, 0, 0));
      ModelShading.setColor(sAxes2, new Color(100, 0, 0));
      Position sAxes1_p = new Position(sAxes1);
      Position sAxes2_p = new Position(sAxes2);
      // Add the shorter axes to the Scene.
      scene.addPosition(sAxes1_p, sAxes2_p);

      // Create a Model of a square.
      Model square = new Model("Example_04d");
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.magenta,
                      Color.green);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Add the square to the Scene.
      Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Rotate the square around its center, with its center at the origin.
      Matrix save  = square_p.matrix;  // save the coordinate system
      Matrix save1 = sAxes1_p.matrix;
      Matrix save2 = sAxes2_p.matrix;
      for (int i = 0; i < 60; i++)
      {
         sAxes2_p.matrix.mult( Matrix.translate(-0.5, -0.5, 0) );
         square_p.matrix.mult( Matrix.translate(-0.5, -0.5, 0) ); // update the coordinate system

         sAxes1.visible = true;
         sAxes2.visible = true;
         square.visible = false;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04d_Frame%03da.ppm", i));

         square.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04d_Frame%03db.ppm", i));

         sAxes1_p.matrix = save1;    // restore the coordinate system
         sAxes2_p.matrix = save2;
         square_p.matrix = save;

         sAxes1.visible = true;
         sAxes2.visible = false;
         square.visible = false;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04d_Frame%03dc.ppm", i));

         sAxes1_p.matrix.mult( Matrix.rotateZ(6) );  // accumulate a rotation
         sAxes2_p.matrix.mult( Matrix.rotateZ(6) );
         square_p.matrix.mult( Matrix.rotateZ(6) );

         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04d_Frame%03dd.ppm", i));

         save1 = sAxes1_p.matrix;    // save the coordinate system again
         save2 = sAxes2_p.matrix;
         save  = square_p.matrix;
      }
   }
}
