/*
   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html#mode=edit
*/

public class VisualizeMatrixOperations_ver2
{
   public static void main(String[] args)
   {
      Mat2 m1 = new Mat2(2, 0, 0, 2);
      Mat2 m2 = new Mat2(3, 0, 0, 3);
      Mat2 m3 = new Mat2(4, 0, 0, 4);

      // m5 = m3 * m2 * m1              // use times()
      Mat2 m5 = m3.times(m2).times(m1); // why m5 and not m4?
           m5 = m3.times(m2);           // 4th matrix
                     m5 = m5.times(m1); // 5th matrix

      // m0 = m3 * m2 * m1              // use mult()
      Mat2 m0 = (new Mat2(1,0,0,1)).mult(m3).mult(m2).mult(m1);
           m0 = m3.mult(m2).mult(m1);  // Bad idea! (Why?)
           m0 = m3.mult(m2.mult(m1));  // Even worse! (Why?)

      System.out.println( m1 );
      System.out.println( m2 );
      System.out.println( m3 );
      System.out.println( m5 );
      System.out.println( m0 );
   }
}

class Vec2
{
   public double x, y;

   public Vec2(double x, double y)
   {
      this.x = x;
      this.y = y;
   }

   public Vec2 plus(Vec2 v)
   {
      return new Vec2( x+(v.x), y+(v.y) );
   }

   public String toString(){return "["+x+",\t"+y+"]";}
}


class Mat2
{
   private Vec2 v1, v2; // these are column vectors

   public Mat2(double m00, double m01, double m10, double m11)
   {
      this.v1 = new Vec2(m00, m10); // 1st column
      this.v2 = new Vec2(m01, m11); // 2nd column
   }

   public Mat2 times(Mat2 m)
   {
      Vec2 vec1 = this.times(m.v1);
      Vec2 vec2 = this.times(m.v2);
      return new Mat2(vec1.x, vec2.x, vec1.y, vec2.y);
   }

   public Mat2 mult(Mat2 m)
   {
      Vec2 vec1 = this.times(m.v1);
      Vec2 vec2 = this.times(m.v2);

      this.v1 = vec1;
      this.v2 = vec2;
      return this; // for method chaining
   }

    public Vec2 times(Vec2 v)
    {
       double x = (v1.x * v.x) + (v2.x * v.y);
       double y = (v1.y * v.x) + (v2.y * v.y);
       return new Vec2(x, y);
   }

   public String toString(){return v1+"\n"+v2+"\n";}
}

/*
Notice the following important failing of the Java language.

The times() method has this signature,

   public Mat2 times(Mat2 m)

and it is applied like this,

   m3.times(m2).times(m1)

The mult() method has essentially the same signature,

   public Mat2 mult(Mat2 m)

and it is applied in essentially the same way,

   m3.mult(m2).mult(m1)

So there is nothing in the Java syntax, nor in the Java
type system, that helps you realize that mult() mutates
the object it is called on, while times() returns a new
object. You can only tell the difference between the
two methods by reading the details of their source code.

Another way to put this is that a method's interface,
its signature,

   public Mat2 times(Mat2 m)
   public Mat2  mult(Mat2 m)

tells you very little about what the method does. The
interface of a method does not work as a specification.
In some modern functional programming languages (such as
Haskell) interfaces are much better at being specifications.
*/
