/*
   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html#mode=edit

   Compare the code below with this simple example.
      int[] a = {1,2,3,4,5};
      for (int n : a) {
         n = 2*n;
      }
*/

public class VisualizeMatrixVectorOperations
{
   public static void main(String[] args)
   {
      java.util.List<Vec2> vectorList =
         java.util.Arrays.asList(
            new Vec2(1,1),
            new Vec2(2,2),
            new Vec2(3,3));
      System.out.println( vectorList );

      Mat2 m = new Mat2(2, 0, 0, 3);

      // Update vectorList, version 1?
      for (Vec2 v : vectorList) {
         v = m.times(v);  // Doesn't work!
      }
      System.out.println( vectorList );

      // Update vectorList, version 2.
      for (Vec2 v : vectorList) {
         v.set( m.times(v) ); // v = M*v
      }
      System.out.println( vectorList );

      // Update vectorList, version 3.
      for (Vec2 v : vectorList) {
         v.mult(m); // Kind of misleading; looks like v = v*M.
      }
      System.out.println( vectorList );

      // Update vectorList, version 4.
      for (Vec2 v : vectorList) {
         m.mult(v); // use m to mutate v
      }
      System.out.println( vectorList );

      // Update vectorList, version 5.
      for (int i = 0; i < vectorList.size(); i++)
      {
         vectorList.set(i, m.times(vectorList.get(i))); // Unreadable.
      }
      System.out.println( vectorList );

      // Update vectorList, version 6.
      java.util.List<Vec2> tempVecs = new java.util.ArrayList<>();
      for (Vec2 v : vectorList)
      {
         tempVecs.add( m.times(v) );
      }
      vectorList = tempVecs;
      System.out.println( vectorList );

      // Update vectorList, version 7.
      // This version is really sily! Why?
      tempVecs = new java.util.ArrayList<>();
      for (Vec2 v : vectorList)
      {
         tempVecs.add( v.set( m.times(v) ) );
      }
      vectorList = tempVecs;
      System.out.println( vectorList );
   }
}


class Mat2
{
   public double m00, m01, m10, m11;

   public Mat2(double m00, double m01, double m10, double m11) {
      this.m00 = m00;
      this.m01 = m01;
      this.m10 = m10;
      this.m11 = m11;
   }

   public Vec2 times(Vec2 v) {
      return new Vec2(this.m00 * v.x + this.m01 * v.y,
                      this.m10 * v.x + this.m11 * v.y);
   }

   public Vec2 mult(Vec2 v) { // mutate v and return a reference to it
      v.x = this.m00 * v.x + this.m01 * v.y;
      v.y = this.m10 * v.x + this.m11 * v.y;
      return v;  // for method chaining
   }
}

class Vec2
{
   public double x, y;

   public Vec2(double x, double y) {
      this.x = x;
      this.y = y;
   }

   public Vec2 set(Vec2 v) {
      this.x = v.x;
      this.y = v.y;
      return this;  // for method chaining
   }

   public Vec2 mult(Mat2 m) {
      double x = m.m00 * this.x + m.m01 * this.y;
      double y = m.m10 * this.x + m.m11 * this.y;
      this.x = x;
      this.y = y;
      return this;  // for method chaining
   }

   public String toString(){return "["+x+", "+y+"]";}
}

/*
There are three ways to update a list in Java.
 1) You can mutate each of the elements referred to by the List.
 2) You can mutate the List object to make it refer to new elements.
 3) You can mutate the List reference variable to refer to a new List.

Here is a picture of a Java List

    List             ArrayList          +------+
  reference           object      +---->|"cat" | object
   +----+            +-------+    |     +------+
   | *--|----------->|  *----|----+
   +----+            +-------+        +------+
                     |  *----|------->|"dog" | object
                     +-------+        +------+
                     |  *----|-----+
                     +-------+     |        +------+
                     |  *----|--+  +------->|"car" | object
                     +-------+  |           +------+
                                |   +------+
                                +-->|"pan" | object
                                    +------+

One way to update this list is to mutate each object in the list.

    List             ArrayList          +------+
  reference           object      +---->|"cats"|
   +----+            +-------+    |     +------+
   | *--|----------->|  *----|----+
   +----+            +-------+        +------+
                     |  *----|------->|"dogs"|
                     +-------+        +------+
                     |  *----|-----+
                     +-------+     |        +------+
                     |  *----|--+  +------->|"cars"|
                     +-------+  |           +------+
                                |   +------+
                                +-->|"pans"|
                                    +------+

On the other hand, you can mutate the List object so that it
refers to new objects (and let the previous, unchanged, objects
possibly be garbage collected).

                                   +------+
                               +-->|"cats"|
                               |   +------+
    List             ArrayList |        +------+
  reference           object   |        |"cat" |
   +----+            +-------+ |        +------+    +------+
   | *--|----------->|  *----|-+  +---------------->|"dogs"|
   +----+            +-------+    |   +------+      +------+
                     |  *----|----+   |"dog" |
                     +-------+        +------+          +------+
                     |  *----|------------------------->|"cars"|
                     +-------+              +------+    +------+
                     |  *----|--+           |"car" |
                     +-------+  |           +------+
                                |   +------+
                                |   |"pan" |
                                |   +------+
                                |     +------+
                                +---->|"pans"|
                                      +------+

Or you can create a new List object that refers to new objects and
mutate the List reference variable to refer to the new List object.

    List             ArrayList          +------+
  reference           object      +---->|"cat" |
   +----+            +-------+    |     +------+
   | *--|-----+      |  *----|----+
   +----+     |      +-------+        +------+
              |      |  *----|------->|"dog" |
              |      +-------+        +------+
              |      |  *----|-----+
              |      +-------+     |        +------+
              |      |  *----|--+  +------->|"car" |
              |      +-------+  |           +------+
              |                 |   +------+
              |                 +-->|"pan" |
              |                     +------+
              |
              |      ArrayList          +------+
              |       object      +---->|"cats"|
              |      +-------+    |     +------+
              +----->|  *----|----+
                     +-------+        +------+
                     |  *----|------->|"dogs"|
                     +-------+        +------+
                     |  *----|-----+
                     +-------+     |        +------+
                     |  *----|--+  +------->|"cars"|
                     +-------+  |           +------+
                                |   +------+
                                +-->|"pans"|
                                    +------+

In the vector list code examples above, determine which
examples are using which of these techniques.
*/
