/*

*/

package renderer.pipeline;
import  renderer.scene.*;

import java.util.List;
import java.util.ArrayList;

/**
   Transform each {@link Vertex} of a {@link Model} from the
   model's (private) local coordinate system to the {@link Camera}'s
   (shared) view coordinate system.
<p>
   For each {@code Vertex} object in a {@code Model} object,
   use a {@link Position}'s model {@link Matrix} to transform the
   {@code Vertex} object's coordinates from the model's coordinate
   system to the camera's view coordinate system.
*/
public class Model2View
{
   /**
      Use a {@link Position}'s model {@link Matrix} to transform each
      {@link Vertex} from a {@link Model}'s coordinate system to the
      {@link Camera}'s view coordinate system.

      @param model        {@link Model} with {@link Vertex} objects in the model's coordinate system
      @param modelMatrix  a {@link Position}'s transformation {@link Matrix}
      @return a new {@link Model} with {@link Vertex} objects in the camera's view coordinate system
   */
   public static Model model2view(final Model model, final Matrix modelMatrix)
   {
      // A new vertex list to hold the transformed vertices.
      final List<Vertex> newVertexList = new ArrayList<>(model.vertexList.size());

      // Replace each Vertex object with one that contains
      // view coordinates.
      for (final Vertex v : model.vertexList)
      {
         newVertexList.add( modelMatrix.times(v) );
      }

      return new Model(newVertexList,
                       model.lineSegmentList,
                       model.colorList,
                       model.name,
                       model.visible,
                       model.debug);
   }
}
